/* A shooting game: step3 　2015-07-01 by Kosei Demura　*/

#include "dm3.h"
#include <time.h>
#include <math.h>
#include <stdlib.h>

#define TARGET_NUM 10   // 敵の数
#define BULLET_NUM 100  // 弾数

int bullet_no = 0;
int target_hit[TARGET_NUM] = {0}; // 弾が当たったら１に設定
double start_x =0.0, start_y = 0.5, start_z = 0.5; // 初期位置
double R[12] = {1,0,0,0, 0,1,0,0, 0,0,1,0}; // 回転行列が格納される配列, 位置(x,y,z)[m]
double target_pos[TARGET_NUM][3] = {0.0, 0.0, 0.5};   // 位置(x,y,z)[m]
double robot_pos[3]  = {start_x, start_y, start_z}; // ロボットの位置
double bullet_pos[BULLET_NUM][3]; // 弾の位置;
double sides[3] = {1.0, 0.5, 1.0}; // 直方体のサイズ(x, y, z)[m]

void setBullet(int no, double pos[3]) {
    bullet_pos[no][0] = pos[0];
    bullet_pos[no][1] = pos[1];
    bullet_pos[no][2] = pos[2];
}

void command(int cmd) {
    switch (cmd) {
    case 'a':
        robot_pos[0] -= 1.0;
        break;
    case 'd':
        robot_pos[0] += 1.0;
        break;
    case 'x':
        robot_pos[1] -= 1.0;
        break;
    case 'w':
        robot_pos[1] += 1.0;
        break;
    case 32:
        setBullet(bullet_no,robot_pos);
        bullet_no++;
        printf("bullet=%d\n",bullet_no);
        break;
    default:
        printf("Input %c (%d)\n",(char)cmd, cmd);
    }
}

void drawTarget() {
    int i;
    for (i = 0; i < TARGET_NUM; i++) {
        if (target_hit[i]) continue;
        dsSetColor(1.0, 0.0, 0.0); // 色の設定
        target_pos[i][0] = 2 * i  + start_x - TARGET_NUM + 1;  // 位置のx成分
        target_pos[i][1] = start_y + 10;       // 位置のx成分
        target_pos[i][2] = 0.5;                 // 位置のz成分
        dsDrawBox(target_pos[i],R,sides); // 直方体の表示
    }
}

void drawBullet(double pos[3]) {
    int i;
    dsSetColor(0.0, 0.0, 1.0); // 色の設定
    dsDrawSphere(pos,R,0.1); // 球の表示
}

// ブロックが他のブロックと衝突したら１を返す，それ以外は０を返す
int collision(double pos1[3], double pos2[3]) {
    //　ブロック間の距離が１未満なら１を返すコードを
    //  ここに書きなさい
    double dist = sqrt((pos1[0]-pos2[0]) * (pos1[0]-pos2[0])
                       + (pos1[1]-pos2[1]) * (pos1[1]-pos2[1]));
    if (dist < 0.5) return 1;
    else            return 0;
}

void simLoop(int pause) {      /***  シミュレーションループ　***/
    int i,j;
    // ロボットの表示
    dsSetColor(1.0, 1.0, 0.0);   // 黄色
    dsDrawSphere(robot_pos, R, 0.5);

    for (i=0; i < bullet_no; i++) {
        bullet_pos[i][1] += 0.01;
        for (j = 0; j < TARGET_NUM; j++) {
            if (collision(bullet_pos[i],target_pos[j])) {
                target_hit[j] = 1;
            }
        }
        drawBullet(bullet_pos[i]);  // 弾の描画
    }
    drawTarget();  // 敵の描画
}

int main() {       /*** main関数 ***/
    robot_pos[1] = start_y - 10;
    dmLoop(800, 600); // シミュレーションループ ウインドウの幅，高
    return 0;
}
